@@ -49,6 +49,7 @@ from huggingface_hub.inference._providers.openai import OpenAIConversationalTask
 from huggingface_hub.inference._providers.ovhcloud import OVHcloudConversationalTask
 from huggingface_hub.inference._providers.publicai import PublicAIConversationalTask
 from huggingface_hub.inference._providers.replicate import (
+    ReplicateAutomaticSpeechRecognitionTask,
     ReplicateImageToImageTask,
     ReplicateTask,
     ReplicateTextToSpeechTask,
@@ -62,7 +63,8 @@ from huggingface_hub.inference._providers.wavespeed import (
     WavespeedAITextToImageTask,
     WavespeedAITextToVideoTask,
 )
-from huggingface_hub.inference._providers.zai_org import ZaiConversationalTask
+from huggingface_hub.inference._providers.zai_org import _POLLING_INTERVAL as ZAI_POLLING_INTERVAL
+from huggingface_hub.inference._providers.zai_org import ZaiConversationalTask, ZaiTextToImageTask
 
 from .testing_utils import assert_in_logs
 
@@ -397,7 +399,7 @@ class TestFalAIProvider:
     def test_automatic_speech_recognition_response(self):
         helper = FalAIAutomaticSpeechRecognitionTask()
         response = helper.get_response({"text": "Hello world"})
-        assert response == "Hello world"
+        assert response == {"text": "Hello world"}
 
         with pytest.raises(ValueError):
             helper.get_response({"text": 123})
@@ -1475,6 +1477,74 @@ class TestOVHcloudAIEndpointsProvider:
 
 
 class TestReplicateProvider:
+    def test_automatic_speech_recognition_payload(self):
+        helper = ReplicateAutomaticSpeechRecognitionTask()
+
+        mapping_info = InferenceProviderMapping(
+            provider="replicate",
+            hf_model_id="openai/whisper-large-v3",
+            providerId="openai/whisper-large-v3",
+            task="automatic-speech-recognition",
+            status="live",
+        )
+
+        payload = helper._prepare_payload_as_dict(
+            "https://example.com/audio.mp3",
+            {"language": "en"},
+            mapping_info,
+        )
+
+        assert payload == {"input": {"audio": "https://example.com/audio.mp3", "language": "en"}}
+
+        mapping_with_version = InferenceProviderMapping(
+            provider="replicate",
+            hf_model_id="openai/whisper-large-v3",
+            providerId="openai/whisper-large-v3:123",
+            task="automatic-speech-recognition",
+            status="live",
+        )
+
+        audio_bytes = b"dummy-audio"
+        encoded_audio = base64.b64encode(audio_bytes).decode()
+
+        payload = helper._prepare_payload_as_dict(
+            audio_bytes,
+            {},
+            mapping_with_version,
+        )
+
+        assert payload == {
+            "input": {"audio": f"data:audio/wav;base64,{encoded_audio}"},
+            "version": "123",
+        }
+
+    def test_automatic_speech_recognition_get_response_variants(self, mocker):
+        helper = ReplicateAutomaticSpeechRecognitionTask()
+
+        result = helper.get_response({"output": "hello"})
+        assert result == {"text": "hello"}
+
+        result = helper.get_response({"output": ["hello-world"]})
+        assert result == {"text": "hello-world"}
+
+        result = helper.get_response({"output": {"transcription": "bonjour"}})
+        assert result == {"text": "bonjour"}
+
+        result = helper.get_response({"output": {"translation": "hola"}})
+        assert result == {"text": "hola"}
+
+        mock_session = mocker.patch("huggingface_hub.inference._providers.replicate.get_session")
+        mock_response = mocker.Mock(text="file text")
+        mock_response.raise_for_status = lambda: None
+        mock_session.return_value.get.return_value = mock_response
+
+        result = helper.get_response({"output": {"txt_file": "https://example.com/output.txt"}})
+        mock_session.return_value.get.assert_called_once_with("https://example.com/output.txt")
+        assert result == {"text": "file text"}
+
+        with pytest.raises(ValueError):
+            helper.get_response({"output": 123})
+
     def test_prepare_headers(self):
         helper = ReplicateTask("text-to-image")
         headers = helper._prepare_headers({}, "my_replicate_key")
@@ -1841,6 +1911,151 @@ class TestZaiProvider:
         url = helper._prepare_url("hf_token", "test-model")
         assert url.startswith("https://router.huggingface.co/zai-org")
 
+    def test_text_to_image_prepare_route(self):
+        helper = ZaiTextToImageTask()
+        route = helper._prepare_route("glm-image", "zai_token")
+        assert route == "/api/paas/v4/async/images/generations"
+
+    def test_text_to_image_prepare_headers(self):
+        helper = ZaiTextToImageTask()
+        headers = helper._prepare_headers({}, "test_key")
+        assert headers["Accept-Language"] == "en-US,en"
+        assert headers["x-source-channel"] == "hugging_face"
+
+    def test_text_to_image_prepare_url(self):
+        helper = ZaiTextToImageTask()
+        assert helper.task == "text-to-image"
+        url = helper._prepare_url("zai_token", "glm-image")
+        assert url == "https://api.z.ai/api/paas/v4/async/images/generations"
+
+        # Test with HF token (should route through HF proxy)
+        url = helper._prepare_url("hf_token", "glm-image")
+        assert url.startswith("https://router.huggingface.co/zai-org")
+
+    def test_text_to_image_prepare_payload(self):
+        helper = ZaiTextToImageTask()
+        payload = helper._prepare_payload_as_dict(
+            "A cute cat sitting on a sunny windowsill",
+            {"width": 1280, "height": 1280},
+            InferenceProviderMapping(
+                provider="zai-org",
+                hf_model_id="zai-org/glm-image",
+                providerId="glm-image",
+                task="text-to-image",
+                status="live",
+            ),
+        )
+        assert payload == {
+            "model": "glm-image",
+            "prompt": "A cute cat sitting on a sunny windowsill",
+            "size": "1280x1280",
+        }
+
+    def test_text_to_image_prepare_payload_no_size(self):
+        helper = ZaiTextToImageTask()
+        payload = helper._prepare_payload_as_dict(
+            "A cute cat",
+            {},
+            InferenceProviderMapping(
+                provider="zai-org",
+                hf_model_id="zai-org/glm-image",
+                providerId="glm-image",
+                task="text-to-image",
+                status="live",
+            ),
+        )
+        assert payload == {
+            "model": "glm-image",
+            "prompt": "A cute cat",
+        }
+
+    def test_text_to_image_get_response_success(self, mocker):
+        helper = ZaiTextToImageTask()
+        mock_session = mocker.patch("huggingface_hub.inference._providers.zai_org.get_session")
+        mock_sleep = mocker.patch("huggingface_hub.inference._providers.zai_org.time.sleep")
+
+        # Mock polling response and image download
+        mock_session.return_value.get.side_effect = [
+            # First call: poll for status (still processing)
+            mocker.Mock(
+                json=lambda: {"task_status": "PROCESSING", "id": "8353992347972780031"},
+                raise_for_status=lambda: None,
+            ),
+            # Second call: poll for status (success)
+            mocker.Mock(
+                json=lambda: {
+                    "task_status": "SUCCESS",
+                    "id": "8353992347972780031",
+                    "image_result": [{"url": "https://example.com/image.png"}],
+                },
+                raise_for_status=lambda: None,
+            ),
+            # Third call: download image
+            mocker.Mock(content=b"image_bytes", raise_for_status=lambda: None),
+        ]
+
+        api_key = helper._prepare_api_key("hf_token")
+        headers = helper._prepare_headers({}, api_key)
+        url = helper._prepare_url(api_key, "glm-image")
+
+        request_params = RequestParameters(
+            url=url,
+            headers=headers,
+            task="text-to-image",
+            model="glm-image",
+            data=None,
+            json=None,
+        )
+
+        response = helper.get_response(
+            {"id": "8353992347972780031", "task_status": "PROCESSING", "model": "glm-image"},
+            request_params,
+        )
+
+        assert response == b"image_bytes"
+        assert mock_session.return_value.get.call_count == 3
+        mock_sleep.assert_called_once_with(ZAI_POLLING_INTERVAL)
+
+    def test_text_to_image_get_response_immediate_success(self, mocker):
+        """Test when the response is already successful (no polling needed)."""
+        helper = ZaiTextToImageTask()
+        mock_session = mocker.patch("huggingface_hub.inference._providers.zai_org.get_session")
+
+        mock_session.return_value.get.return_value = mocker.Mock(content=b"image_bytes", raise_for_status=lambda: None)
+
+        response = helper.get_response(
+            {
+                "id": "8353992347972780031",
+                "task_status": "SUCCESS",
+                "image_result": [{"url": "https://example.com/image.png"}],
+            },
+            None,
+        )
+
+        assert response == b"image_bytes"
+        mock_session.return_value.get.assert_called_once_with("https://example.com/image.png")
+
+    def test_text_to_image_get_response_fail(self):
+        helper = ZaiTextToImageTask()
+        with pytest.raises(ValueError, match="ZAI image generation failed"):
+            helper.get_response(
+                {"id": "8353992347972780031", "task_status": "FAIL"},
+                None,
+            )
+
+    def test_text_to_image_get_response_no_task_id(self):
+        helper = ZaiTextToImageTask()
+        with pytest.raises(ValueError, match="No task_id in response"):
+            helper.get_response({}, None)
+
+    def test_text_to_image_get_response_no_image_result(self, mocker):
+        helper = ZaiTextToImageTask()
+        with pytest.raises(ValueError, match="No image_result in response"):
+            helper.get_response(
+                {"id": "8353992347972780031", "task_status": "SUCCESS"},
+                None,
+            )
+
 
 class TestBaseConversationalTask:
     def test_prepare_route(self):
